﻿Type=Class
Version=2.52
@EndOfDesignText@

Sub Class_Globals

	Public cColors    As compCls
    Public cMaterials As compCls
    Public cMeshes    As compCls
    Public cNodes     As compCls
    Public cScene     As compCls
    Public cCamera    As compCls
    Public cInput     As compCls
	
	Public useHighContrastPalette As Boolean = True
	Private mm As mainCls 'NEW
End Sub

Public Sub Initialize(mm0 As mainCls) As Boolean

	mm = mm0
	init_components
End Sub

Private Sub init_components()

    cColors.Initialize   (1, "Colors")
    cMaterials.Initialize(2, "Materials")
    cMeshes.Initialize   (3, "Meshes")
    cNodes.Initialize    (4, "Nodes")
    cScene.Initialize    (5, "Scene")
    cCamera.Initialize   (6, "Camera")
    cInput.Initialize    (7, "Input")
End Sub

Public Sub create_components() As Boolean

	mm.colors1.Initialize
    mm.materials.Initialize
    mm.meshes.Initialize
    mm.nodes.Initialize
    mm.shapes.Initialize
    mm.scene.Initialize
    mm.camera1.Initialize
    mm.window.Initialize
    mm.engine.Initialize(mm)
    mm.gpu.Initialize(mm)
    mm.input.Initialize
    Return True
End Sub

Public Sub build_component(c As compCls) As Boolean

    Dim s As String = c.id & ". Build " & c.name & ".."
    Log(s)

    Select Case c.id
        Case cColors.id:    bld_colors
        Case cMaterials.id: bld_materials
        Case cMeshes.id:    bld_meshes
        Case cScene.id:     bld_scene
        Case cNodes.id:     bld_nodes
        Case cCamera.id:    bld_camera
    End Select

    Log(s & " ok.")
    Return True
End Sub

Public Sub bld_colors() As Boolean

    Dim cs As colorsCls = mm.colors1
    Return True
End Sub

Public Sub bld_materials() As Boolean 'call this before shapes-builder!

    Dim ms As materialsCls = mm.materials
    Dim cs As colorsCls = mm.colors1
    Dim sides As Int = 2        'values: 1, 2; Use 2 for glass-effect
    Dim alpha As Float = 1.0       'values: 0.0 - 1.0; Use 0.5 for glass-effect
    Dim c6, c7, c8, c9 As Int

    ms.setMaterial(ms.iRubber1, 0.0, 1.0, 2, 0, 0, 0): ms.material(ms.iRubber1).width = 2
    ms.setMaterial(ms.iWood1, 0.3, 1.0, 1, 12, 12, 6) 
    ms.setMaterial(ms.iMetal1, 0.8, 1.0, 1, 6, 6, 8)
    ms.setMaterial(ms.iRoad1, 0.2, 1.0, 1, 11, 11, 11)
    ms.setMaterial(ms.iHullSphere, 0.0, 0.5, 1, 9, 13, 12)

    If useHighContrastPalette Then
         c6 = 1: c7 = 1: c8 = 2: c9 = 3
    Else: c6 = 6: c7 = 7: c8 = 8: c9 = 9
    End If

    ms.setMaterial(ms.iBlack, 0.0, alpha, sides, 0, 0, 0)
    ms.setMaterial(ms.iWhite, 0.0, alpha, sides, 15, 15, 15)
    ms.setMaterial(ms.iGray1, 0.0, alpha, sides, 13, 13, 13)
    ms.setMaterial(ms.iGray2, 0.0, alpha, sides, 11, 11, 11)
    ms.setMaterial(ms.iGreen1, 0.0, alpha, sides, c9, 15, c9)
    ms.setMaterial(ms.iLemon1, 0.0, alpha, sides, 13, 13, c8)
    ms.setMaterial(ms.iOrange1, 0.0, alpha, sides, 15, 12, c8)
    ms.setMaterial(ms.iOrange2, 0.0, alpha, sides, 14, 11, c7)
    ms.setMaterial(ms.iMagenta1, 0.0, alpha, sides, 15, c6, 12)
    ms.setMaterial(ms.iViolet1, 0.0, alpha, sides, 12, c6, 15)
    ms.setMaterial(ms.iCyan1, 0.0, alpha, sides, c8, 13, 15)
    Return True
End Sub

Public Sub bld_meshes() As Boolean  'NOTE: Depends on materials!

	Dim ms As meshesCls = mm.meshes
    Dim m1 As mesh3 = ms.mesh(ms.iCube1)
    Dim m2 As mesh3 = ms.mesh(ms.iCrate1)
    Dim m3 As mesh3 = ms.mesh(ms.iCrate2)
    Dim m4 As mesh3 = ms.mesh(ms.iCrate3)
    Dim m5 As mesh3 = ms.mesh(ms.iCask1)
    Dim m6 As mesh3 = ms.mesh(ms.iChecker1)
    Dim m7 As mesh3 = ms.mesh(ms.iChecker2)
    Dim m8 As mesh3 = ms.mesh(ms.iChecker3)
    Dim m9 As mesh3 = ms.mesh(ms.iSphere1)

    mm.meshlab.build_crate (m1, 16, 16, 16)
    mm.meshlab.build_crate (m2, 16,  4, 24)
    mm.meshlab.build_crate (m3,  8, 20, 16)
    mm.meshlab.build_crate (m4, 32, 32,  32)
    mm.meshlab.build_cask (m5, 11, 16, 32)
    mm.meshlab.build_checkers4 (m6, 4, 24)
    mm.meshlab.build_checkers3 (m7, 12, 43)
    mm.meshlab.build_checkers3 (m8, 2, 256)
    'mm.meshlab.build_sphere_1 (m9, 16, 24)
    mm.meshlab.build_sphere_2 (m9, 16)

    mm.meshes.finishAll()
    Return True
End Sub

Public Sub bld_scene() As Boolean 'NOTE: Relies on scene and meshes! 'scene = New scene3()

    'scene.init(100)           'while not really needed, this may speed up starting process (recommended values: 100-1000) 
    Return True
End Sub

Public Sub bld_nodes() As Boolean

    mm.nodes.reset
    Dim node As node3

    node = mm.nodes.node_add2()
    If node.idx >= 0 Then
        node.reset(9)
        add_shapes2_ground(node)
    End If

    node = mm.nodes.node_add2()
    If node.idx >= 0 Then
        node.reset(12)
        add_shapes2_bodies(node)
    End If        
    Return True
End Sub

Public Sub bld_shapes1() As Boolean
    
    Dim n1 As Int = 3
    Dim n2 As Int = 2
    Dim x, z As Int
    Dim cx As Double = 0.0
    Dim cy As Double = 0.0
    Dim cz As Double = 0.0
    Dim sy As Float
    Dim ms As meshesCls = mm.meshes
    Dim shape As shape3

    mm.shapes.reset
    mm.shapes.stypeUse = mm.shapes.stypeGround  'replace sshapes.set_type (shape.idx, 2)
    cy = 0.0

    For z = -n1 To n1
        For x = -n1 To n1
            cx = x * 108
            cz = z * 108
            shape = mm.shapes.shape_add2()
            If shape.active Then 
                shape.mesh_load (ms.mesh(ms.iChecker1))
                mm.shapes.move_to (shape.idx, cx, cy, cz)
            End If
        Next
    Next

    mm.shapes.stypeUse = mm.shapes.stypeBodies  'replaces shapes.set_type (shape.idx, 1)

    For z = -n2 To n2
        For x = -n2 To n2
            cx = x * 80
            cz = z * 80
            shape = mm.shapes.shape_add2()
            If shape.active Then 
                shape.mesh_load (ms.mesh(ms.iCask1))
                sy = 3 / (Sqrt(x * x + z * z) + 1)
                shape.mesh.rescale (1.0, sy, 1.0)
                cy = shape.mesh.radiusInner
                mm.shapes.move_to (shape.idx, cx, cy, cz)
            End If
        Next
    Next

    mm.shapes.stypeUse = mm.shapes.stypeBodies
    cx = 0.0
    cz = -320.0

    shape = mm.shapes.shape_add2()
    If shape.active Then
        shape.mesh_load (ms.mesh(ms.iSphere1))
        cy = shape.mesh.radiusInner
        'shape.mesh.rescale (1.0, 1.0, 1.0)
        shape.form.moveTo (cx, cy, cz)
    End If

    Return True
End Sub

Private Sub add_shapes2_ground(node As node3) As Boolean
    
    Dim n1 As Int = 1
    Dim x, z As Int
    Dim cx, cy, cz As Double
    Dim ms As meshesCls = mm.meshes
    Dim shape As shape3

    mm.shapes.reset
    mm.shapes.stypeUse = mm.shapes.stypeGround
    cy = 0.0

    For z = -n1 To n1
        For x = -n1 To n1
            cx = x * 512
            cz = z * 512
            shape = mm.shapes.shape_add2()
            If shape.active Then
                If z = 0 AND x = 0 Then
                     shape.mesh_load (ms.mesh(ms.iChecker2))
                     shape.stype = mm.shapes.stypeBodies
                Else: shape.mesh_load (ms.mesh(ms.iChecker3))
                      shape.stype = mm.shapes.stypeGround
                End If
                mm.shapes.move_to (shape.idx,  cx, cy, cz)
                shape.show_radius = False

                node.shape_add (shape.idx)
            End If
        Next
    Next
    Return True
End Sub

Private Sub add_shapes2_bodies(node As node3) As Boolean
    
    Dim c As Int
    Dim cx, cy, cz As Double
    Dim a1, a2 As Double
    Dim ms As meshesCls = mm.meshes
    Dim shape As shape3

    mm.shapes.stypeUse = mm.shapes.stypeBodies
    cx = 0.0
    cz = -320.0
    a1 = 0
    a2 = cPI*2 / 12

    For c = 1 To 12
        cx = Cos(a1) * 120
        cz = Sin(a1) * 120: a1 = a1 + a2
        shape = mm.shapes.shape_add2()
        If shape.active Then
            shape.mesh_load (ms.mesh(ms.iSphere1))
            cy = shape.mesh.radiusInner * 1.0
            shape.form.moveTo (cx, cy, cz)
            shape.show_radius = True

            node.shape_add (shape.idx)
        End If
    Next
    Return True
End Sub

Public Sub bld_camera() As Boolean 'camera = New camera3()

    mm.camera1.setPlanes (1, 1000)
    mm.camera1.setLens (200)
    Return True
End Sub

