﻿Type=Class
Version=2.52
@EndOfDesignText@

Sub Class_Globals
	
	Public idx As Int   'sg = statically generated data

	Public radiusInner As Float  'sg: mesh hull radius
	Public radiusOuter As Float  'sg: mesh hull radius

	Public fnum As Int, lnum As Int, pnum As Int
	Public fmax As Int, lmax As Int, pmax As Int ', cmate As Int 'currently used material
	Public fmateUse As Int
	Public lmateUse As Int

	Public fp1(), fp2(), fp3(), fp4() As Int 'shape face data ->
	'Public fr1(), fr2() As Float
	Public fcx(), fcy(), fcz() As Float 'sg: face center-point
	Public fnx(), fny(), fnz() As Float 'sg: face normal-vector / eye-vector
	Public fux(), fuy(), fuz() As Float 'sg: face up-vector (NEW)
	Public ftype() As Int   '0=Null, 1=Dot, 2=Bar, 3=Tri, 4=Quad, 5=Sphere(BB), 6=Square(BB), 7=Rect(BB)
	Public fmate() As Int   'face material (references to global materialList)

	Public px(), py(), pz() As Float 'shape point.xyz

	Public lp1() As Int     'shape line p1, not used yet
	Public lp2() As Int     'shape line p2, not used yet
	Public lmate() As Int   'line material
End Sub

Public Sub Initialize() As Boolean

    idx = -1  'means mesh not initialized/assigned yet
    pnum = 0
    lnum = 0
    fnum = 0  
End Sub
          
Public Sub init(ptsMax As Int, lnsMax As Int, fcsMax As Int) 'array size adapted to b4a

    Dim pm As Int = ptsMax: pmax = pm: pnum = 0
    Dim lm As Int = lnsMax: lmax = lm: lnum = 0: lmateUse = 0
    Dim fm As Int = fcsMax: fmax = fm: fnum = 0: fmateUse = 0       

    Dim fp1(fm), fp2(fm), fp3(fm), fp4(fm) As Int
    'ReDim fr1(fm), fr2(fm)
    Dim fcx(fm), fcy(fm), fcz(fm) As Float
    Dim fnx(fm), fny(fm), fnz(fm) As Float
    Dim fux(fm), fuy(fm), fuz(fm) As Float
    Dim ftype(fm) As Int
    Dim fmate(fm) As Int

    Dim px(pm), py(pm), pz(pm) As Float
    
    Dim lp1(lm), lp2(lm) As Int
    Dim lmate(lm) As Int
End Sub

Public Sub finish()

    createHullspheres
    createCenterNormals
End Sub

Public Sub rescale(scx As Float, scy As Float, scz As Float)

    Dim p As Int

    For p = 0 To pnum-1
        px(p) = px(p) * scx
        py(p) = py(p) * scy
        pz(p) = pz(p) * scz
    Next
    finish
End Sub
   
Private Sub createCenterNormals()

    Dim f As Int = 0
    Dim t As Int
    Dim p1, p2, p3, p4 As Int

    Do While (f < fnum)

        t = ftype(f)
        p1 = fp1(f): p2 = fp2(f): p3 = fp3(f): p4 = fp4(f)

        Select Case t
            Case 1: createCenter2(f, p1, p2)
            Case 2: createCenter2(f, p1, p2)
            Case 3: createCenter3(f, p1, p2, p3)
                    createNormal3(f, p1, p2, p3)
            Case 4: createCenter4(f, p1, p2, p3, p4)
                    createNormal3(f, p1, p2, p3)
        End Select
        
        'shiftCenterByNormal(i, -8) 'improves face priorty problem (z-sort, painters algorithm)
        f = f + 1
    Loop
End Sub

Private Sub createHullspheres()

    Dim i As Int = 0
    Dim ri As Float = 0 'inner hull radius
    Dim ro As Float = 0 'outer hull radius
    Dim x, y, z, r As Float

    Do While (i < pnum)
        x = Abs(px(i)): If x > ri Then ri = x
        y = Abs(py(i)): If y > ri Then ri = y
        z = Abs(pz(i)): If z > ri Then ri = z
        r = Sqrt(x * x + y * y + z * z): i = i + 1
        If r > ro Then ro = r
    Loop
    radiusInner = ri
    radiusOuter = ro
End Sub

Public Sub setPoint(i As Int, x As Float, y As Float, z As Float)
  
    If checkIndex(1, i) Then
        px(i) = x
        py(i) = y
        pz(i) = z
    End If
End Sub

Public Sub setLine(i As Int, p1 As Int, p2 As Int)
  
    If checkIndex(2, i) Then
        lp1(i) = p1
        lp2(i) = p2
        lmate(i) = lmateUse
    End If
End Sub

Public Sub setFace1(i As Int, p As Int, eye As point3, up As point3)
  
    If checkIndex(3, i) Then
        resetFace(i)
        fp1(i) = p: fp2(i) = p
        fnx(i) = eye.x: fux(i) = up.x
        fny(i) = eye.y: fuy(i) = up.y
        fnz(i) = eye.z: fuz(i) = up.z
        ftype(i) = 1
        fmate(i) = fmateUse
    End If
End Sub

Public Sub setFace2(i As Int, p1 As Int, p2 As Int, eye As point3)
  
    If checkIndex(3, i) Then
        resetFace(i)
        fp1(i) = p1: fp2(i) = p2
        fnx(i) = eye.x: fux(i) = 0.0
        fny(i) = eye.y: fuy(i) = 0.0
        fnz(i) = eye.z: fuz(i) = 0.0
        ftype(i) = 2
        fmate(i) = fmateUse
    End If
End Sub

Public Sub setFace3(i As Int, p1 As Int, p2 As Int, p3 As Int)
  
    If checkIndex(3, i) Then
        resetFace(i)
        fp1(i) = p1: fp2(i) = p2: fp3(i) = p3
        ftype(i) = 3
        fmate(i) = fmateUse
    End If
End Sub

Public Sub setFace4(i As Int, p1 As Int, p2 As Int, p3 As Int, p4 As Int)

    If checkIndex(3, i) Then
        resetFace(i)
        fp1(i) = p1: fp2(i) = p2: fp3(i) = p3: fp4(i) = p4
        ftype(i) = 4
        fmate(i) = fmateUse
    End If
End Sub

Public Sub setLineMate(mi As Int)   'aka. setLineMaterial
    lmateUse = mi
End Sub
Public Sub setFaceMate(mi As Int)   'aka. setFaceMaterial
    fmateUse = mi
End Sub

'Public Sub useMaterial_REPLACED(mi As Int)
'    cmate = mi
'End Sub
'-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Private Sub resetFace(f As Int)

    fp1(f) = 0: fp2(f) = 0: fp3(f) = 0: fp4(f) = 0
    'fr1(f) = 0: fr2(f) = 0
    fcx(f) = 0: fnx(f) = 0
    fcy(f) = 0: fny(f) = 0
    fcz(f) = 0: fnz(f) = 0
End Sub

'check item-index and update item-number
Private Sub checkIndex(item As Int, i As Int) As Boolean

    Dim ok As Boolean = False

    Select Case item
        Case 1: ok = (i >= 0 OR i < pmax)     'item 1: check point index
                If pnum <= i Then pnum = i+1
        Case 2: ok = (i >= 0 OR i < lmax)     'item 2: check line index
                If lnum <= i Then lnum = i+1
        Case 3: ok = (i >= 0 OR i < fmax)     'item 3: check face index
                If fnum <= i Then fnum = i+1  
    End Select
    Return ok
End Sub

'create an unified normal (length = 1)
Private Sub createNormal3(f As Int, p1 As Int, p2 As Int, p3 As Int)

    Dim x1 As Double = px(p1) - px(p2)
    Dim y1 As Double = py(p1) - py(p2)
    Dim z1 As Double = pz(p1) - pz(p2)
    Dim x2 As Double = px(p3) - px(p2)
    Dim y2 As Double = py(p3) - py(p2)
    Dim z2 As Double = pz(p3) - pz(p2)

    Dim x As Double = y1 * z2 - y2 * z1
    Dim y As Double = z1 * x2 - z2 * x1
    Dim z As Double = x1 * y2 - x2 * y1
    Dim r As Double = Sqrt(x * x + y * y + z * z)
    Dim s As Double = 0: If r > 0.0001 Then s = 1 / r

    'Debug.Assert (s > 0.001)

    fnx(f) = x * s
    fny(f) = y * s
    fnz(f) = z * s
End Sub

Private Sub createCenter2(f As Int, p1 As Int, p2 As Int)

    fcx(f) = (px(p1) + px(p2)) / 2
    fcy(f) = (py(p1) + py(p2)) / 2
    fcz(f) = (pz(p1) + pz(p2)) / 2
End Sub

Private Sub createCenter3(f As Int, p1 As Int, p2 As Int, p3 As Int)

    fcx(f) = (px(p1) + px(p2) + px(p3)) / 3
    fcy(f) = (py(p1) + py(p2) + py(p3)) / 3
    fcz(f) = (pz(p1) + pz(p2) + pz(p3)) / 3
End Sub

Private Sub createCenter4(f As Int, p1 As Int, p2 As Int, p3 As Int, p4 As Int)

    fcx(f) = (px(p1) + px(p2) + px(p3) + px(p4)) / 4
    fcy(f) = (py(p1) + py(p2) + py(p3) + px(p4)) / 4
    fcz(f) = (pz(p1) + pz(p2) + pz(p3) + px(p4)) / 4
End Sub

'Can improve face depth-sorting results. Recommended value = 8
Private Sub shiftCenterByNormal_UNUSED(f As Int, u As Float)

    fcx(f) = fcx(f) + fnx(f) * u
    fcy(f) = fcy(f) + fny(f) * u
    fcz(f) = fcz(f) + fnz(f) * u
End Sub