﻿Type=Class
Version=2.52
@EndOfDesignText@

Sub Class_Globals

	Public ax() As Float 'ax,ay,az = current axis system (3x4 matrix represents the pos-axes-system)
	Public ay() As Float
	Public az() As Float 

	Private xc1, yc1, zc1 As Float
	Private xc2, yc2, zc2 As Float
	Private xc3, yc3, zc3 As Float
	Private xd1, yd1, zd1 As Float
	Private xd2, yd2, zd2 As Float
	Private xd3, yd3, zd3 As Float
End Sub

Public Sub Initialize() As Boolean 'array size adapted to b4a

    Dim ax(4), ay(4), az(4) As Float
    reset
End Sub

Public Sub reset() 'loads the identity matrix

    ax(0) = 0: ay(0) = 0: az(0) = 0  'position
    ax(1) = 1: ay(1) = 0: az(1) = 0  'axis-1: x-axis
    ax(2) = 0: ay(2) = 1: az(2) = 0  'axis-2: y-axis
    ax(3) = 0: ay(3) = 0: az(3) = 1  'axis-3: z-axis
End Sub

Public Sub moveTo(px As Double, py As Double, pz As Double)

    ax(0) = px
    ay(0) = py
    az(0) = pz
End Sub

Public Sub rotateAbs(ang1 As Double, ang2 As Double, ang3 As Double)
    
    reset
    rotate(ang1, ang2, ang3)
End Sub

Public Sub rotate(ang1 As Double, ang2 As Double, ang3 As Double)
    
    Dim r1 As Float = ang1 * cPI / 180.0
    Dim r2 As Float = ang2 * cPI / 180.0
    Dim r3 As Float = ang3 * cPI / 180.0

    rotatePoint(1, r1, r2, r3)
    rotatePoint(2, r1, r2, r3)
    rotatePoint(3, r1, r2, r3)
End Sub

'Rotate a Point around the Axes
Private Sub rotatePoint(p As Int, r1 As Double, r2 As Double, r3 As Double)

    Dim rc, rs As Float
    Dim x, y, z As Double
    
    If r3 <> 0 Then
        x = ax(p): rc = Cos(r3)
        y = ay(p): rs = Sin(r3)       
        ax(p) = x * rc + y * rs
        ay(p) = -x * rs + y * rc
    End If
    If r2 <> 0 Then
        x = ax(p): rc = Cos(r2)
        z = az(p): rs = Sin(r2)
        ax(p) = x * rc + z * rs
        az(p) = -x * rs + z * rc
    End If
    If r1 <> 0 Then
        y = ay(p): rc = Cos(r1)
        z = az(p): rs = Sin(r1)
        az(p) = z * rc + y * rs
        ay(p) = -z * rs + y * rc
    End If
End Sub

Public Sub rotate2 (f1 As Float, f2 As Float, f3 As Float)

    Dim f As Float
    Dim xd, yd, zd As Float

    xc1 = ax(1): yc1 = ay(1): zc1 = az(1)
    xc2 = ax(2): yc2 = ay(2): zc2 = az(2)
    xc3 = ax(3): yc3 = ay(3): zc3 = az(3)

    'calc x-axis movement
    xd1 = xc2 * f3 - xc3 * f2
    yd1 = yc2 * f3 - yc3 * f2
    zd1 = zc2 * f3 - zc3 * f2
    'calc y-axis movement
    xd2 = xc3 * f1 - xc1 * f3
    yd2 = yc3 * f1 - yc1 * f3
    zd2 = zc3 * f1 - zc1 * f3
    'calc z-axis movement
    xd3 = xc1 * f2 - xc2 * f1
    yd3 = yc1 * f2 - yc2 * f1
    zd3 = zc1 * f2 - zc2 * f1

    'add movements
    xc1 = xc1 + xd1: yc1 = yc1 + yd1: zc1 = zc1 + zd1
    xc2 = xc2 + xd2: yc2 = yc2 + yd2: zc2 = zc2 + zd2
    xc3 = xc3 + xd3: yc3 = yc3 + yd3: zc3 = zc3 + zd3

    'must correct vector-rotation error
    'Z-Axis
    'unify Z-Axis length
    f = 1 / Sqrt(xc3 * xc3 + yc3 * yc3 + zc3 * zc3)
    xc3 = xc3 * f: yc3 = yc3 * f: zc3 = zc3 * f
    'X-Axis
    'make X-Axis rightangled to Z-Axis -> create X to Z-Axis diagonal vector
    xd = xc1 - xc3: yd = yc1 - yc3: zd = zc1 - zc3
    'scale length to squareroot(2)
    f = 1.414214 / Sqrt(xd * xd + yd * yd + zd * zd)
    xd = xd * f: yd = yd * f: zd = zd * f
    xc1 = xc3 + xd: yc1 = yc3 + yd: zc1 = zc3 + zd
    'correct unprecision: unify X-Axis length
    f = 1 / Sqrt(xc1 * xc1 + yc1 * yc1 + zc1 * zc1)
    xc1 = xc1 * f: yc1 = yc1 * f: zc1 = zc1 * f
    'Y-Axis
    'construct Y-axis by crossproduct of X- and Z-Axis (=normal vector to XZ-Plane)
    xc2 = -(yc1 * zc3 - zc1 * yc3)
    yc2 = -(zc1 * xc3 - xc1 * zc3)
    zc2 = -(xc1 * yc3 - yc1 * xc3)

    ax(1) = xc1: ay(1) = yc1: az(1) = zc1 
    ax(2) = xc2: ay(2) = yc2: az(2) = zc2 
    ax(3) = xc3: ay(3) = yc3: az(3) = zc3 
End Sub

Public Sub moveZ (f3 As Float)

    ax(0) = ax(0) + ax(3) * f3
    ay(0) = ay(0) + ay(3) * f3
    az(0) = az(0) + az(3) * f3
End Sub

Public Sub lout()

    Log ("xform.ax0 " & sout(0))
    Log ("xform.ax1 " & sout(1))
    Log ("xform.ax2 " & sout(2))
    Log ("xform.ax3 " & sout(3))
End Sub

Private Sub sout(a As Int) As String

	NumberFormat(ax(a), 1, 2)

    Return "(" & NumberFormat(ax(a), 1, 2) & "|" & _
                 NumberFormat(ay(a), 1, 2) & "|" & _
                 NumberFormat(az(a), 1, 2) & ") "
End Sub
